%% 
% this file was downloaded from 
% http://white.stanford.edu/~knk/Psych216A/Psych216ALecture5Tutorial.m
% It is an excellent and detailed exposition of the topic
% Note that the program uses crossvalidind.m function from
% MATLAB's Neural Network Toolbox

%%
% Psych 216A Tutorial 4: Model accuracy
%
% This matlab tutorial is intended to complement lecture #5.
%
% We will show how to:
%
% (1) use cross-validation to estimate R2 and reduce data overfitting.
%
% (2) Use cross-validation to estimate an R2 for two models and select the
%     most accurate model.
%
% Written by Franco Pestilli, PhD, Stanford University, May 2012
%
% frk@stanford.edu

% Models are attempts to describe data. Ideally a model's description of
% the data should be accurate and reliable. This would allow for making
% predictions on data that we have not collected.
%
% Accuracy and reliability are two important descriptors of the quality of
% a model.
%
% Accuracy: A model is accurate if the predictions it makes are %close to
%           measured data. The coefficient of determination (R2) is a standard
%           measure of model accuracy. R2 expresses the percent of variance
%           in the data explained by the model.
%
% Reliability: A model is reliable if the predictions (and the parameters)
%              of the model vary minimally as the sample data used to fit
%              the model changes. The standard deviation (or the confidence
%              intervals) of a model's estimates is a good measure of
%              model reliability.
%
% Model accuracy and reliability can be estimated using cross-validation
% and bootstrapping, respectively.
%
% In cross-validation, we fit a model to a subset of the total measured
% data and then we evaluate how well the model *predicts* the remaining
% data.
%
% In bootstrapping, we resample (with replacement) from the original data
% %set and then fit the model of interest to each bootstrap sample. This
% allows us to quantify reliability; how much the model prediction changes
% as samples change.
%
% The present Tutorial focuses on model accuracy. Tutorial 6 will focus on
% model reliability.

%% Use cross-validation to estimate model accuracy, and reduce overfitting.
%
% The goal of fitting data to a sample is that of describing an underlying
% population that we cannot access. The only thing we have is the data in
% the sample. 
% 
% Unfortunately, every time we compute a model on a sample of data the
% model will overfit the sample. This means that the model will adjust "too
% much" to the specific sample, so that the model will be a better
% description of the sample than of the population we are interested in.
%
% Here after we will show how to use cross-validation to estimate model
% accuracy by computing a measure of R2 that is not prone to overfitting.
%
% We will create a population using a straight line as model. We will then
% create a sample out of this population. We will then fit a straight line
% model to the sample.
%
% We will then estimate the accuracy of our model using the coefficient of
% determination (R2). R2 represent the percent of variance in the sample
% data explained by the model.
%
% Because we are using a good model (straight line), a straight line. The
% model will be accurate, its predictions will be %close to the sample data.
% In other terms, it will represent the data very well.
% 
% But the model we fit is tailored to the specific sample we had collected.
% When fitting a (good) model to a sample limited in size we are prone to
% overfitting. This means that the model will tend to describe the specific
% sample better than it would describe the true population. In other terms
% the R2 estimated on the sample will be higher that should be.
%
% We will show how to use cross-validation to compute a coefficient of
% determination (R2) that is not prone to overfitting.
clear all, clc

rng(10) %for reproducibility

% We will first simulate a population. Using a "true" model of our choice;
% a straight line.
x = repmat(1:200,100,1)';
x = x + randn(size(x)) .* 5;
a = 2; b = 100; % parameters of the model.
polynomialOrder  = 1; % A straight line
y = polyval([a b],x,polynomialOrder) + randn(size(x)) .* 70;

% Show the simulated true population.
close all,
h = figure(1);
set(h,'Name','Use cross-validation to reduce overfitting')
plot(x,y,'ro','MarkerFaceColor','r')
xlabel('x');
ylabel('y')

%%

% Show the true model on top of the population.
polynomialOrder  = 1; % A straight line.
true_m = polyval([a b],x,polynomialOrder);
hold on
plot(x,true_m,'k-', 'LineWidth',4)

%%
% We will now sample from the population.
sampleSize = 20; numObservations = 2;
ind1 = randsample(1:size(x,1),sampleSize);
ind2 = randsample(1:size(x,2),numObservations);
x_sample = x(ind1,ind2);
y_sample = y(ind1,ind2);

% Show the sample.
plot(x_sample,y_sample,'co','MarkerFaceColor','c','MarkerSize',12)

%%

% Let's fit a model to the sample. A straight-line model.
polynomialOrder  = 1; 
ab = polyfit(x_sample,y_sample,polynomialOrder);

% Evaluate the estimated model.
y_fit = polyval(ab,x_sample);

% Show the model fit to the sample.
plot(x_sample,y_fit,'b-','LineWidth',4)

%%

% We can now compute the coefficient of determination (R2) of the fitted
% model to the sample data. This number shows the percent of variance in
% the data explained by the model.
R2 = 100 * (1 - (sum((y_sample(:) - y_fit(:)).^2) / sum((y_sample(:) - mean(y_sample(:))).^2)));

% The percent of explained variance seem to be decent (it should be above
% 70%). But we know that some overfitting is going on here. 
%
% To show this we will now compute the R2 of the true model to the sample
% data. This value wll be lower then the previous R2, indicating that the
% fitted model does not describe the true population as well as it
% describes the given sample. The improved accuracy of the fitted model
% on the sample data is called overfitting.
y_fit_truem = polyval([a b],x_sample);
R2true  = 100 .* (1 - (sum((y_sample(:) - y_fit_truem(:)).^2) / sum((y_sample(:) - mean(y_sample(:))).^2)));

% We can use cross-validation to reduce overfitting.
%
% To do so, we will use leave-one-out cross validation and split the sample
% data into two sets, a training- and a testing-%set. We will fit the
% straight-line model to the "training %set" and compute the R2 on the "test
% %set." This procedures reduces overfitting because the model prediction is
% never made on the same sample used for fitting. So any excessive
% tailoring of the model to the training sample will reduced the quality of
% the predicion on the test sample.
sampleSize      = size(y_sample,1);
numFolds        = 200 ;
R2crossValidate = 0;
numToLeaveOut = 1; % number of samples to leave out on each fold
all_test_y = []; all_y_hat = [];
for i_fold = 1:numFolds
 
 % We split the sample into training and tests sub-samples.
 % We use Matlab's crossvalin.m.
 [train,test] = crossvalind('LeaveMOut',sampleSize,numToLeaveOut);
 
 train_x = x_sample(train,:);
 train_y = y_sample(train,:);
 
 test_x  = x_sample(test,:);
 test_y  = y_sample(test,:);
 
 % We fit the model to the training %set
 ab_hat = polyfit(train_x,train_y,1);
 
 % We make a prediction with the test_x and the model parameters just
 % fitted
 y_hat  = polyval(ab_hat, test_x);
 
 all_test_y = [all_test_y, test_y];
 all_y_hat  = [all_y_hat,   y_hat];
 
 % Show the fits of the current model.
 plot(test_x,y_hat,'g.', 'MarkerSize',25);
 drawnow
end

% We compute the R2
R2crossValidate = 100 * (1 - (sum((all_test_y - all_y_hat).^2) / ...
 sum((all_test_y - mean(all_test_y)).^2)));

title(sprintf('R^2 estimated from the sample: %2.3f,\nR^2 cross-validated: %2.3f\n', ...
 R2,R2crossValidate))

fprintf('(1) The R^2 estimated on the sample is: %2.3f\n',R2)
fprintf('(2) The R^2 estimated on the population is: %2.3f\n',R2true)
fprintf('(3) The cross-validated R^2 is: %2.3f\n',R2crossValidate)

% You might noticed that the cross-validated R2 (3) can be smaller than
% either the R2 computed on the original sample (1) or R2 of the true
% population model to the sample data (2). This is because the cross
% validation R2 is computed on a subset of the data (smapleSize - 1 in our
% case). So each eastimate is a noisier than the estimate on the full
% sample.
%
% Cross-validation can be used to reduce overfitting. But the exact value
% of the R2 obtained via cross-validation will depends on a series of
% factors:
% 
% 1. the true noise level in the population.
% 2. the size of your sample.
% 3. the complexty of your model, i.e., how many parameters.
%
% So all the cross-valdiated R2 really tells us is the accuracy of the
% model in describing the population without overfitting. What we cannot
% say is whether the exact value of cross-validated R2 is similar to the
% population R2.


%%
% Use cross-validation to select the best of two models.
%
% Here we will show how to use cross-vlidation to select the best of two
% models.
%
% Ideally, the model that has a higher R2 is better or more accurate at
% describing the data.
%
% But, if we were to compare how accurate two different models are and we
% were to judge model accuracy by using the non cross-validated R2, we
% could be erroneously selecting the wrong mpdel. For example we could
% select a model that has higher-degree of overfitting. A model that
% overfits the data more will have a higher non cross-validated R2 on the
% sample data %set.
%
% So, when interested in selecting the best among several models we should
% always use cross-validated R2. These R2 evaluate the accuracy of a model
% with out overfitting the data.
%
% Next, I will show how to do this.

% We open a new figure but we use the previous population and sample.
h = figure(2);
set(h,'Name','Use cross-validation to choose the best of two models')
% plot the population
plot(x,y,'ro','MarkerFaceColor','r')
xlabel('x');
ylabel('y')

%%

% Show the true model on top of the population.
hold on
plot(x,true_m,'k-', 'LineWidth',4)

%%

% Show the sample.
plot(x_sample,y_sample,'co','MarkerFaceColor','c','MarkerSize',12)

%%

% We now show the fit of a linear model we obtaned above.
plot(x_sample,y_fit,'b-','LineWidth',4)

%%

% Now let's fit a more complex model. A plolynomial of 4th order.
% This model is not similar to a the true model (a straight line), but will
% represent better the specific data in the sample better than a straight
% line, because it will be able to adapt better to the small variability in
% the data in the sample due to chance.
polyOrder  = 4;
params = polyfit(x_sample,y_sample,polyOrder);

% %plot the estimated model.
y_fitP4 = polyval(params,x_sample);
[~, i] = sort(x_sample);
plot(x_sample(i),y_fitP4(i),'m-','LineWidth',4)

%%

% The polynomial of 4th order (magenta) passes closer to each sample data
% point (cyan) than the straight line (blue) does. It shuld have a higher
% R2 than the straight-line model.
R2_line = 100 * (1 - (sum((y_sample(:) - y_fit(:)).^2) / sum((y_sample(:) - mean(y_sample(:))).^2)));
R2_p4 = 100 * (1 - (sum((y_sample(:) - y_fitP4(:)).^2) / sum((y_sample(:) - mean(y_sample(:))).^2)));

%%
% So if we where to judge the accuracy of these two models from the
% computed R2, we would select the polynomial of order 4. But what we are
% really interested in is to choose the model that best represet the true
% population. So, choosing the polynomial of order 4 would be a mistake.
% (Remember, that we created the population from a straight line model.)

% To choose the best model we need a measure of model accuracy that is not
% prone to overfitting. We have seen above that cross validation allows us
% to reduce overfitting. So we will now compute the cross-validate R2 for
% both models and show that after taking care of overfitting the polynomial
% does not fit the data better anymore.
sampleSize      = size(y_sample,1);
numFolds        = 200 ;
R2crossValidate = 0;
n = 1; % number of samples to leave out on each fold
all_test_y = []; all_y_hatlin = []; all_y_hatp = [];
for i_fold = 1:numFolds 
 % We split the sample into training and testing halves.
 [train,test] = crossvalind('LeaveMout',sampleSize,n);

 train_x = x_sample(train,:);
 train_y = y_sample(train,:);
 
 test_x  = x_sample(test,:);
 test_y  = y_sample(test,:);
 
 all_test_y = [all_test_y, test_y];

 % We fit the linear model to the training %set:
 ab_hatl = polyfit(train_x,train_y,1);
 
 % We predict y from the test_x and the model parameters just fitted
 y_hat         = polyval(ab_hatl, test_x);
 all_y_hatlin  = [all_y_hatlin,   y_hat];
 
 % We fit the polynomial model to the training %set:
 ab_hatp = polyfit(train_x,train_y,polyOrder);
 
 % We predict y from the test_x and the model parameters just fitted
 y_hatp      = polyval(ab_hatp, test_x);
 all_y_hatp  = [all_y_hatp,   y_hatp];
 
 % Show the fits of the current model.
 plot(test_x,y_hat,'b+', 'MarkerSize',10);
 plot(test_x,y_hatp,'m+', 'MarkerSize',10);

 drawnow
end

% We compute the R2
R2crossValidate_lin = 100 * (1 - (sum((all_test_y - all_y_hatlin).^2) / ...
 sum((all_test_y - mean(all_test_y)).^2)));

% We compute the R2
R2crossValidate_poly = 100 * (1 - (sum((all_test_y - all_y_hatp).^2) / ...
 sum((all_test_y - mean(all_test_y)).^2)));

fprintf('\n(A) The R^2 for the straight-line model is: %2.3f\n',R2_line)
fprintf('(B) The R^2 for the polynomial of order 4 is: %2.3f\n',R2_p4)
fprintf('(C) The cross-validated R^2 for the straight-line model is: %2.3f\n',R2crossValidate_lin)
fprintf('(D) The cross-validated R^2 for the polynomial of order 4 is: %2.3f\n',R2crossValidate_poly)

% The polynomial had a higher degree of overfitting. This can be seen by
% comparing the R2 of the polynomial with the cross-validated R2. The
% larger the difference the larger the overfitting. 
%
% This means that polynomial is not a more accurate model in describing the
% data than a straight line, but instead it overfits the sample more.