% day4

%%
% Simple Ordinary Least Squares (OLS)
%generate data
rng('default')
nSample     = 100;
nRegressor  = 5;
beta        = round(10*randn(nRegressor+1,1));
regX        = randn(nSample,nRegressor);
X           = [ones(nSample,1) regX];
sigma_eps   = 10;
epsilon     = sigma_eps*randn(nSample,1);
% y follows y = X*beta + epsilon
y           = X*beta + epsilon;

% (a) calculate beta manually
betaManual  = inv(X'*X)*X'*y;

% (b) more efficient way
betaShorter = X\y;

% (c) beta using LeSage's econx toolbox function ols
resLeSage   = ols(y,X);
betaLeSage  = resLeSage.beta;

% (d) beta using regress function from Stats and ML toolbox
[betaReg,bintReg,rReg,rintReg,statsReg] = regress(y,X);


% compare betas
[beta betaManual betaShorter betaLeSage betaReg]

% compute residuals
residuals = y - X*betaManual;

% compare standard errors of beta
constCorrection = nSample/(nSample-(nRegressor+1));
bstdManual = sqrt(constCorrection*diag(inv(X'*X)*var(residuals,1)));
[resLeSage.bstd bstdManual]

% compare t-statistics
[resLeSage.tstat betaManual./bstdManual]

% compare R^2
rsqrManual = 1 - residuals'*residuals/((y-mean(y))'*(y-mean(y)));
[statsReg(1) resLeSage.rsqr rsqrManual]

% compare Confidence intervals
constCI = tinv(0.975,nSample-(nRegressor+1)); 
bintManual = [betaManual - constCI*bstdManual, betaManual + constCI*bstdManual];
[bintReg bintManual]

constCI = tinv(0.975,nSample);  
bintManual = [betaManual - constCI*bstdManual, betaManual + constCI*bstdManual];
[resLeSage.bint bintManual]
% notice that LeSage's toolbox uses degrees of freedom uncorrected for small sample
% for CI


%%
% perform a maximum likelihood estimator

logLik = @(beta,X,y) sum((y-X*beta).^2);
    
f = @(beta)logLik(beta,X,y);

options = optimoptions(@fminunc,'Algorithm','quasi-newton');
beta0 = ones(nRegressor+1,1); % Starting guess
[betaML,fval,exitflag,output] = fminunc(f,beta0,options);

betaML


%%
% example from http://www.mathworks.com/help/stats/regress.html
load carsmall
x1 = Weight;
x2 = Horsepower;    % Contains NaN data
y = MPG;

X = [ones(size(x1)) x1 x2 x1.*x2];
b = regress(y,X);    % Removes NaN data

scatter3(x1,x2,y,'filled')
hold on
x1fit = min(x1):100:max(x1);
x2fit = min(x2):10:max(x2);
[X1FIT,X2FIT] = meshgrid(x1fit,x2fit);
YFIT = b(1) + b(2)*X1FIT + b(3)*X2FIT + b(4)*X1FIT.*X2FIT;
mesh(X1FIT,X2FIT,YFIT)
xlabel('Weight')
ylabel('Horsepower')
zlabel('MPG')
view(50,10)

%%
% Instrumental variable regression

rng('default')
nSample     = 10000;
nRegressor  = 2;
beta        = round(10*randn(nRegressor+1,1));
regX        = randn(nSample,nRegressor);
X           = [ones(nSample,1) regX];
sigma_eps   = 10;
epsilon     = sigma_eps*randn(nSample,1);
% create endogenous variable
endoX       = 0.2*epsilon.^2 - 1.2*epsilon;
X           = [X endoX];
corrcoef(endoX,epsilon)
% the correlation is -0.63
% now we create an instrument by removing the part of endoX
% that is explained by epsilon
betaEndo    = [ones(nSample,1) epsilon]\endoX;
instrument  = endoX - [ones(nSample,1) epsilon]*betaEndo;
corrcoef(instrument,epsilon)
corrcoef(instrument,endoX)
% this is a good instrument by construction
y           = X*[beta;1] + epsilon;
betaOLS     = X\y;
% because of the negative correlation the coefficient of endoX
% is downward biased (notice it does not change with larger nSample)
Z           = [X(:,1:(end-1)),instrument];
PZ          = Z*inv(Z'*Z)*Z';
betaIV      = inv(X'*PZ*X)*X'*PZ*y;

% or alternatively (algebraically identical)
Xhat        = PZ*X;
betaIV2     = inv(Xhat'*Xhat)*Xhat'*y;

% notice we get the coefficients right
[[beta;1] betaOLS betaIV]


%%
% Panel data

% see examples at www.paneldatatoolbox.com
% note that syntax of the ols function is different from the LeSage's toolbox

%%
% Bootstrap - rolling dices

nSimul  = 1000;
nSample = 12;
sample = [1,6,6,6,6,6,4,5,2,3,4,4,4];
meanSample = sum(sample)/nSample;

rng(11101985)
meansSimulated = zeros(nSimul,1);

for iSimul = 1:nSimul
    meansSimulated(iSimul) = sum(sample(randi([1,nSample],nSample,1)))/nSample;
end

mean(meansSimulated > 3.5)

%%
% Bootstrap example
% [requires Stats and ML toolbox]
% http://www.mathworks.com/help/stats/bootstrp.html
% Grade point average and LSAT scores from 15 law schools


load lawdata
rng default  % For reproducibility
[bootstat,bootsam] = bootstrp(1000,@corr,lsat,gpa);

bootsam(:,1:5)
figure
hist(bootstat)

ksdensity(bootstat)

%%
rng default;  % For reproducibility
y = exprnd(ones(100,1));
m = bootstrp(1000,@mean,y);

%Plot an estimate of the density of these bootstrapped means.
figure;
[fi,xi] = ksdensity(m);
plot(xi,fi);

%%
rng default;
load hald
% Perform a linear regression and compute the residuals.

x = [ones(size(heat)),ingredients];
y = heat;
b = regress(y,x);
yfit = x*b;
resid = y - yfit;

%Estimate the standard errors by bootstrapping the residuals.
se = std(bootstrp(1000,@(bootr)regress(yfit+bootr,x),resid));



%%
% PCA - Principal components analysis
% http://www.mathworks.com/help/stats/quality-of-life-in-u-s-cities.html

load cities
categories
figure()
boxplot(ratings,'orientation','horizontal','labels',categories)

C = corr(ratings,ratings);

%maximal correlation
max(max(abs(C-eye(9))))

%we scale the data by their variance (they have different units etc.)
w = 1./var(ratings);
[wcoeff,score,latent,tsquared,explained] = pca(ratings); %,'VariableWeights',w);

c3 = wcoeff(:,1:3);
coefforth = inv(diag(std(ratings)))*wcoeff;
cscores = zscore(ratings)*coefforth;


figure()
plot(score(:,1),score(:,2),'+')
xlabel('1st Principal Component')
ylabel('2nd Principal Component')

gname
latent
explained

figure()
pareto(explained)
xlabel('Principal Component')
ylabel('Variance Explained (%)')

%%
% PCA - simulated example

rng('default')
x1 = randn(100,1);
x2 = 0.4*x1 + 0.001*randn(100,1);
x3 = -0.5*x1 + 0.01*randn(100,1);
x4 = 0.1*x1.^2 + 0.01*randn(100,1);

X  = [x1 x2 x3 x4];

[wcoeff,score,latent,tsquared,explained] = pca(X);

%%
% Cross-validation
% see crossValidation.m
% by Franco Pestilli, PhD, Stanford University, May 2012
% also here
% http://white.stanford.edu/~knk/Psych216A/Psych216ALecture6Tutorial.m
% is a tutorial that shows how we can use bootstrap to study model
% reliability, it is saved as reliability.m

%%
% SVM - Support Vector Machines

% example here http://www.mathworks.com/help/stats/support-vector-machines-svm.html
rng(1); % For reproducibility
r = sqrt(rand(100,1)); % Radius
t = 2*pi*rand(100,1);  % Angle
data1 = [r.*cos(t), r.*sin(t)]; % Points
%Generate 100 points uniformly distributed in the annulus. The radius is again proportional to a square root, this time a square root of the uniform distribution from 1 through 4.

r2 = sqrt(3*rand(100,1)+1); % Radius
t2 = 2*pi*rand(100,1);      % Angle
data2 = [r2.*cos(t2), r2.*sin(t2)]; % points
%Plot the points, and plot circles of radii 1 and 2 for comparison.

figure;
plot(data1(:,1),data1(:,2),'r.','MarkerSize',15)
hold on
plot(data2(:,1),data2(:,2),'b.','MarkerSize',15)
ezpolar(@(x)1);ezpolar(@(x)2);
axis equal
hold off


data3 = [data1;data2];
theclass = ones(200,1);
theclass(1:100) = -1;
%Train an SVM classifier with KernelFunction set to 'rbf' and BoxConstraint set to Inf. Plot the decision boundary and flag the support vectors.

%Train the SVM Classifier
cl = fitcsvm(data3,theclass,'KernelFunction','rbf',...
    'BoxConstraint',Inf,'ClassNames',[-1,1]);

% Predict scores over the grid
d = 0.02;
[x1Grid,x2Grid] = meshgrid(min(data3(:,1)):d:max(data3(:,1)),...
    min(data3(:,2)):d:max(data3(:,2)));
xGrid = [x1Grid(:),x2Grid(:)];
[~,scores] = predict(cl,xGrid);

% Plot the data and the decision boundary
figure;
h(1:2) = gscatter(data3(:,1),data3(:,2),theclass,'rb','.');
hold on
ezpolar(@(x)1);
h(3) = plot(data3(cl.IsSupportVector,1),data3(cl.IsSupportVector,2),'ko');
contour(x1Grid,x2Grid,reshape(scores(:,2),size(x1Grid)),[0 0],'k');
legend(h,{'-1','+1','Support Vectors'});
axis equal
hold off


% another example here http://www.robots.ox.ac.uk/~az/lectures/ml/matlab2.pdf

%%
% LASSO
% https://www.mathworks.com/help/stats/lasso.html

% Chemical reaction data with correlated predictors
load acetylene
Xs = [x1 x2 x3];
X = x2fx(Xs,'interaction');
X(:,1) = []; % No constant term
%Construct the lasso fit using ten-fold cross validation. Include the FitInfo output so you can plot the result.

[B FitInfo] = lasso(X,y,'CV',10);
%Plot the cross-validated fits.

lassoPlot(B,FitInfo,'PlotType','CV');


%%
% quantile regression
% for large scale problems
% http://web.stanford.edu/~jiyan/quantreg_wp/quantreg.html


%%
% Kernel density estimator

rng('default')  % for reproducibility
x = [randn(30,1); 5+randn(30,1)];

[f,xi] = ksdensity(x);
[f,xi] = ksdensity(x,'bandwidth',0.3);
figure
plot(xi,f);
