function [ out ] = iv2sls( y, X, H, options )
%IV2SLS Instrumental Variables Two Stage Least Squares estimation
%   est = IV2SLS(Y,X,H,OPTIONS) computes instrumental variables 2SLS
%   estimation where Y is the vector of the dependant variable, X is the 
%   matrix of the independent variables, H is the matrix of instruments,
%   and OPTIONS is an optational parameter to specify options.
%   Possible values for OPTIONS are:
%
%    'robust' Compute robust (White) standard errors
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    % Check input
    if nargin < 1
        error('Dependent variable not specified');
    end
    if nargin < 2
        error('Independent variables not specified');
    end
    if size(y,2) ~= 1
        error('Y must be a column vector of the dependant variable');
    end
    if size(y,1) ~= size(X,1)
        error('Number of rows in Y must be equal to number of rows in X');
    end
    if nargin < 3
        error('Instruments not specified');
    end
    if size(y,1) ~= size(H,1)
        error('Number of rows in Y must be equal to number of rows in Y and X');
    end
    if size(H,2) < size(X,2)
        error('Number of instruments H must be greater or equal to number of variables X');
    end
    if nargin < 4
        robust = 0;
    end
    if nargin == 4
        if strcmp(options,'robust')
            robust = 1;
        else
            error('Option %s not available',options);
        end
    end
    
    % Output structure
    out = estoutput();

    % Get number of observations
    n = size(y,1);
    N = n;    
       
    % Add constant
    X = [X ones(N,1)];
    H = [H ones(N,1)];

    % Number of variables and instruments
    k = size(X,2);
    l = size(H,2);
    
    % First Stage
    Xhat = H*((H'*H)\H'*X);

    % Compute estimates: Second Stage
    coef = (Xhat'*X)\Xhat'*y;

    % Fitted values
    yhat = X*coef;
    
    % Residuals
    res = y - yhat;

    % Residual variance
    resvar = (res'*res)./(N);

    % Covariance matrix
    if ~robust
        varcoef = resvar * inv(Xhat'*Xhat);
    else
        % White robust standard errors
        varcoef = inv(Xhat'*X) * (Xhat' * diag(diag(res*res')) * Xhat) * inv(Xhat'*X);
    end

    % Standard error
    stderr = sqrt(diag(varcoef));

    % Statistic
    statistic = coef ./ stderr;
    df_statistic = NaN;
    p_statistic = (1 - normaldist(abs(statistic)))' * 2;

    % Joint significance
    R = [eye(k-1) zeros(k-1,1)];
    q = zeros(k-1,1);
    Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
    jointsig = Wald;
    df_jointsig_n = k-1;
    df_jointsig_d = Inf;
    p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);

    % Goodness of fit
    M0 = eye(N) - 1/(N) * ones(N);
    r2 = 1 - (res'*res)./(y'*M0*y);
    adjr2 = 1 - (N - 1)./(N - k) .* (1 - r2);
    
    % Save output output
    out.method = 'IV2SLS';
    out.method2 = '';
    out.y = y;
    out.X = X;
    out.H = H;
    out.N = N;
    out.n = n;
    out.T = 1;
    out.k = k;
    out.l = l;
    out.constant = 1;
    out.robust = robust;
    out.coef = coef;
    out.varcoef = varcoef;
    out.stderr = stderr;
    out.yhat = yhat;
    out.res = res;
    out.resvar = resvar;
    out.statistic = statistic;
    out.df_statistic = df_statistic;
    out.p_statistic = p_statistic;
    out.jointsig = jointsig;
    out.df_jointsig_n = df_jointsig_n;
    out.df_jointsig_d = df_jointsig_d;
    out.p_jointsig = p_jointsig;
    out.r2 = r2;
    out.adjr2 = adjr2;
    

end

