function [ out ] = ols( y, X, options )
%OLS Ordinary Least Squares estimation
%   est = OLS(Y,X,OPTIONS) computes ordinary least square estimation where Y is the
%   vector of the dependant variable, X is the matrix of the independent
%   variables, and OPTIONS is an optional paramer to specify options.
%   Possible values for OPTIONS are:
%
%    'robust' Compute robust (White) standard errors
%
%   AUTHOR:
%   Inmaculada C. ?lvarez
%   Javier Barbero
%   Jos? L. Zof?o
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    % Check input
    if nargin < 1
        error('Dependent variable not specified');
    end
    if nargin < 2
        error('Independent variables not specified');
    end
    if size(y,2) ~= 1
        error('Y must be a column vector of the dependant variable');
    end
    if size(y,1) ~= size(X,1)
        error('Number of rows in Y must be equal to number of rows in X');
    end
    if nargin < 3
        robust = 0;             
    end
    if nargin == 3

        if strcmp(options,'robust')
            robust = 1;
        elseif strcmp(options,'') || strcmp(options,' ')
            robust = 0;
            % No option
        else
            error('Option %s not available',options);
        end

    end
    
    
    % Output structure
    out = estoutput();

    % Get number of observations
    n = size(y,1);
    N = n;
    
    % Add constant
    X = [X ones(N,1)];

    % Number of variables
    k = size(X,2);

    % Compute estimates
    coef = (X'*X)\X'*y;
    
    % Fitted values
    yhat = X*coef;

    % Residuals
    res = y - yhat;

    % Residual variance
    resvar = (res'*res)./(N-k);

    % Covariance matrix
    if ~robust
        varcoef = resvar * inv(X'*X);
    else
        % White robust standard errors
        varcoef = (N)/(N-k) * inv(X'*X) * (X' * diag(diag(res*res')) * X) * inv(X'*X);
    end

    % Standard error
    stderr = sqrt(diag(varcoef));

    % Statistic
    statistic = coef ./ stderr;
    df_statistic = N-k;
    p_statistic = (1 - tdist(abs(statistic), df_statistic))' * 2;

    % Joint significance
    R = [eye(k-1) zeros(k-1,1)];
    q = zeros(k-1,1);
    Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
    jointsig = Wald / (k-1);
    df_jointsig_n = k-1;
    df_jointsig_d = N-k;
    p_jointsig = 1 - fdist(jointsig, df_jointsig_n, df_jointsig_d);

    % Goodness of fit
    M0 = eye(N) - 1/(N) * ones(N);
    r2 = 1 - (res'*res)./(y'*M0*y);
    adjr2 = 1 - (N - 1)./(N - k) .* (1 - r2);
    
    % Save output output
    out.method = 'OLS';
    out.method2 = '';
    out.y = y;
    out.X = X;
    out.N = N;
    out.n = n;
    out.T = 1;
    out.k = k;
    out.constant = 1;
    out.robust = robust;
    out.coef = coef;
    out.varcoef = varcoef;
    out.stderr = stderr;
    out.yhat = yhat;
    out.res = res;
    out.resvar = resvar;
    out.statistic = statistic;
    out.df_statistic = df_statistic;
    out.p_statistic = p_statistic;
    out.jointsig = jointsig;
    out.df_jointsig_n = df_jointsig_n;
    out.df_jointsig_d = df_jointsig_d;
    out.p_jointsig = p_jointsig;
    out.r2 = r2;
    out.adjr2 = adjr2;
    
    

end

