function [ out ] = ivpanel( y, X, H, T, method )
%PANEL Panel data estimation
%   est = IVPANEL(Y,X,H,T,HMETHOD) computes panel data estimation where Y is the
%   vector of the dependant variable, X is the matrix of the independent
%   variables, H is the matrix of instruments, T is the number of 
%   observations per individual, and method is a string for the estimation
%   method to use.
%   Possible values for METHOD are
%
%    'po' Pooled estimation
%    'fe' Fixed-Effects (Within) estimation
%    'be' Between estimation
%    're' Random-Effects GLS estimation: Swamy and Arora
%    'ec' Error Components estimation
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    % Check input
    if nargin < 1
        error('Dependent variable not specified');
    end
    if nargin < 2
        error('Independent variables not specified');
    end
    if size(y,2) ~= 1
        error('Y must be a column vector of the dependant variable');
    end
    if size(y,1) ~= size(X,1)
        error('Number of rows in Y must be equal to number of rows in X');
    end
    if nargin < 3
        error('Instruments not specified');
    end
    if size(y,1) ~= size(H,1)
        error('Number of rows in Y must be equal to number of rows in Y and X');
    end
    if size(H,2) < size(X,2)
        error('Number of instruments H must be greater or equal to number of variables X');
    end
    if nargin < 4
        error('T not specified');
    end
    if length(T) ~= 1
        error('T must be a scalar indicating the number of observations per group');
    end
    if nargin < 5
        warning('METHOD not specified. Using ''fe'' by default');
        method = 'fe';
    end
    
    % Output structure
    out = estoutput();

    % Get number of observations and groups
    N = size(y,1);
    n = N/T;
    
    % Usefull matrices
    Jbar = ones(T)./T;
    E = eye(T) - Jbar;  % Matrix that compute differences from group mean
    P = kron(eye(n),Jbar); % Compute group means and replicate for all observations in the group
    Q = eye(N) - P; % Compute differences from group means
    
    % Original variables
    yorig = y;
    Xorig = X;
    Horig = H;
    
    % Estimate
    switch(method)
        
        % Pooled estimation
        case 'po'
            
            % Add constant
            X = [X ones(N,1)];
            H = [H ones(N,1)];
            
            % Number of variables and instruments
            k = size(X,2);
            l = size(H,2);
            
            % First Stage
            Xhat = H*((H'*H)\H'*X);
            
            % Compute estimates: Second Stage
            coef = (Xhat'*X)\Xhat'*y;
            
            % Fitted values
            yhat = X*coef;
            
            % Residuals
            res = y - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N);
            
            % Covariance matrix
            varcoef = resvar * inv(Xhat'*X);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic)))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            M0 = eye(N) - 1/(N) * ones(N);
            r2 = 1 - (res'*res)./(y'*M0*y);
            adjr2 = 1 - (N - 1)./(N - k) .* (1 - r2);
            
        % Fixed-Effects (Within) estimation
        case 'fe'
            
            % Add constant and transform model
            Xfe = [Q*X ones(N,1)];
            X = [X ones(N,1)];
            Hfe = [Q*H ones(N,1)];
            H = [H ones(N,1)];
            yfe = Q*y;
            
            % Number of variables and instruments
            k = size(X,2);
            l = size(H,2);
            
            % First Stage
            Xfehat = Hfe*((Hfe'*Hfe)\Hfe'*Xfe);
            
            % Compute estimates: Second Stage
            coef = (Xfehat'*Xfe)\Xfehat'*yfe;
            
            % Compute the correct constant term
            alpha = mean(P*y) - mean(P*X*coef);
            coef(k) = alpha;
            
            % Compute individual effects
            ieffect = P*(y-X*coef);
            
            % Fitted values
            yhat = X*coef + ieffect;
                     
            % Residuals
            res = y - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N-k-n+1);
            
            % Covariance matrix
            varcoef = resvar * inv(Xfehat'*Xfe);
            
            % Compute the correct variance for the constant term
            varcoef(k,k) = mean(P*X)*varcoef*mean(P*X)';
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic)))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            r2 = 1 - (res'*res)./(yfe'*yfe);
            %adjr2 = 1 - (N - 1)./(N - k - n) .* (1 - r2);
            adjr2 = NaN;
                
            
        % Between estimation
        case 'be'
            
            % Add constant
            X = [X ones(N,1)];
            H = [H ones(N,1)];
            
            % Number of variables
            k = size(X,2);
            l = size(H,2);
            
            % Transform model
            ybe = sqrt(T) * P * y;
            Xbe = sqrt(T) * P * X;
            Hbe = sqrt(T) * P * H;
            
            % Get only one observation per group
            ybe = ybe(1:T:end);
            Xbe = Xbe(1:T:end,:);
            Hbe = Hbe(1:T:end,:);

            % First Stage
            Xbehat = Hbe*((Hbe'*Hbe)\Hbe'*Xbe);                     
            
            % Compute estimates: Second Stage
            coef = (Xbehat'*Xbe)\Xbehat'*ybe;
            
            % Fitted values
            yhat = Xbe*coef;
            
            % Residuals
            res = ybe - yhat;
            
            % Residual variance
            resvar = (res'*res)./(n-k);
            
            % Covariance matrix
            varcoef = resvar * inv(Xbehat'*Xbe);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic)))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            temp = corrcoef(ybe,yhat).^2;
            r2 = temp(1,2);
            adjr2 = NaN;
            
        % Random-Effects GLS estimation: Swamy and Arora
        case {'re','ec'}          
                        
            % Get sigm2_v from 'fe' residuals
            estfe = ivpanel(yorig,Xorig,H,T,'fe');
            sigma2_v = estfe.resvar;
            
            % Get sigme2_1 from 'be' residuals
            estbe = ivpanel(yorig,Xorig,H,T,'be');        
            sigma2_1 = estbe.resvar;
            
            % sigma2_u and rho_var
            sigma2_mu = (sigma2_1 - sigma2_v)/T;
            % correct sigma2_u if 0 or < 0
            if sigma2_mu < 0
                sigma2_mu = 0;
                sigma2_mu_corrected = 1;
            else
                sigma2_mu_corrected = 0;
            end
            rho_mu = sigma2_mu /(sigma2_mu + sigma2_v);  
            
            % Theta 
            theta = 1- sqrt(sigma2_v/(T*sigma2_mu + sigma2_v));
            
            % If 'ec' use component instruments
            if strcmp(method,'ec')
                H = [Q*H, P*H];
            end
            
            % Add constant
            X = [X ones(N,1)];
            H = [H ones(N,1)];
            
            % Transform the model
            Qre = eye(N) - theta * P;
            yre = Qre*y;
            Xre = Qre*X;
            Hre = Qre*H;
            
            % Number of variables
            k = size(Xre,2);
            l = size(Hre,2);
            
            % First stage
            Xrehat = Hre*((Hre'*Hre)\Hre'*Xre);
            
            % Compute estimates: Second stage
            coef = (Xrehat'*Xre)\Xrehat'*yre;
            
            % Fitted values
            yhat = Xre*coef;
            
            % Residuals
            res = yre - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N-k);
            
            % Covariance matrix
            varcoef = resvar * inv(Xrehat'*Xre);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic))) * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            W = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = W;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            temp = corrcoef(yhat, yre).^2;
            r2 = temp(1,2);
            adjr2 = NaN;
            
            % Specific output for 're'
            out.theta = theta;
            out.sigma_v = sqrt(sigma2_v);
            out.sigma_1 = sqrt(sigma2_1);
            out.sigma_mu = sqrt(sigma2_mu);
            out.sigma_mu_corrected = sigma2_mu_corrected;
            out.rho_mu = rho_mu;
            
        otherwise
            error('Incorrect method especified');
                        
    end
    
    % Save output
    out.method = 'IVPanel';
    out.method2 = method;
    out.y = y;
    out.X = X;
    out.H = H;
    out.N = N;
    out.n = n;
    out.T = T;
    out.k = k;
    out.l = l;
    out.constant = 1;
    out.robust = 0;
    out.coef = coef;
    out.varcoef = varcoef;
    out.stderr = stderr;
    out.yhat = yhat;
    out.res = res;
    out.resvar = resvar;
    out.statistic = statistic;
    out.df_statistic = df_statistic;
    out.p_statistic = p_statistic;
    out.jointsig = jointsig;
    out.df_jointsig_n = df_jointsig_n;
    out.df_jointsig_d = df_jointsig_d;
    out.p_jointsig = p_jointsig;
    out.r2 = r2;
    out.adjr2 = adjr2;
    


end

