function [ out ] = panel( y, X, T, method, options )
%PANEL Panel data estimation
%   est = PANEL(Y,X,T,METHOD,OPTIONS) computes panel data estimation where Y is the
%   vector of the dependant variable, X is the matrix of the independent
%   variables, T is the number of observations per individual, METHOD
%   is a string for the estimation method to use, and OPTIONS is an optional
%   parameter to specify options.
%   Possible values for METHOD are
%
%    'po' Pooled estimation
%    'fe' Fixed-Effects (Within) estimation
%    'be' Between estimation
%    're' Random-Effects GLS estimation: Swamy and Arora
%
%   Possible varlues for OPTIONS are
%
%    'robust' Compute robust standard errors
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    % Check input
    if nargin < 1
        error('Dependent variable not specified');
    end
    if nargin < 2
        error('Independent variables not specified');
    end
    if size(y,2) ~= 1
        error('Y must be a column vector of the dependant variable');
    end
    if size(y,1) ~= size(X,1)
        error('Number of rows in Y must be equal to number of rows in X');
    end
    if nargin < 3
        error('T not specified');
    end
    if length(T) ~= 1
        error('T must be a scalar indicating the number of observations per group');
    end
    if nargin < 4
        warning('METHOD not specified. Using ''fe'' by default');
        method = 'fe';
    end
    if nargin < 5
        robust = 0;             
    end
    if nargin == 5
        if strcmp(options,'robust')
            robust = 1;
            
            if strcmp(method,'be')
                error('Option robust not available for ''be'' estimation');
            end
        else
            error('Option %s not available',options);
        end
    end
    
    % Output structure
    out = estoutput();

    % Get number of observations and groups
    N = size(y,1);
    n = N/T;
    
    % Usefull matrices
    Jbar = ones(T)./T;
    E = eye(T) - Jbar;  % Matrix that compute differences from group mean
    P = kron(eye(n),Jbar); % Compute group means and replicate for all observations in the group
    Q = eye(N) - P; % Compute differences from group means
    
    % Original variables
    yorig = y;
    Xorig = X;
    
    % Estimate
    switch(method)
        
        % Pooled estimation
        case 'po'
            
            % Add constant
            X = [X ones(N,1)];
            
            % Number of variables
            k = size(X,2);
            
            % Compute estimates
            coef = (X'*X)\X'*y;
            
            % Fitted values
            yhat = X*coef;
            
            % Residuals
            res = y - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N-k);
            
            % Covariance matrix
            if ~robust
                varcoef = resvar * inv(X'*X);
            else
                % White robust standard errors
                varcoef = (N)/(N-k) * inv(X'*X) * (X' * diag(diag(res*res')) * X) * inv(X'*X);
            end
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = N-k;
            p_statistic = (1 - tdist(abs(statistic), df_statistic))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald / (k-1);
            df_jointsig_n = k-1;
            df_jointsig_d = N-k;
            p_jointsig = 1 - fdist(jointsig, df_jointsig_n, df_jointsig_d);
            
            % Goodness of fit
            M0 = eye(N) - 1/(N) * ones(N);
            r2 = 1 - (res'*res)./(y'*M0*y);
            adjr2 = 1 - (N - 1)./(N - k) .* (1 - r2);
            
        % Fixed-Effects (Within) estimation
        case 'fe'
            
            % Add constant and transform model
            Xfe = [Q*X ones(N,1)];
            X = [X ones(N,1)];
            yfe = Q*y;
            
            % Number of variables
            k = size(X,2);
            
            % Compute estimates
            coef = (Xfe'*Xfe)\Xfe'*yfe;
            
            % Compute the correct constant term
            alpha = mean(P*y) - mean(P*X*coef);
            coef(k) = alpha;
            
            % Compute individual effects
            ieffect = P*(y-X*coef);
            
            % Fitted values
            yhat = X*coef + ieffect;
                     
            % Residuals
            res = y - yhat;
            
            % Residual variance
            resvar = (res'*res)./(n*(T-1)-k+1);
            
            % Covariance matrix
            if ~robust
                varcoef = resvar * inv(Xfe'*Xfe);
            else
                total_sum = 0;
                for i=1:1:n
                    s = (i-1)*T+1;
                    e = i*T;
                    total_sum = total_sum + Xfe(s:e,:)'*res(s:e)*res(s:e)'*Xfe(s:e,:);
                end
                % Adjust Std.errors for n clusters
                total_sum = total_sum .* (n/(n-1)) * ((N-1)/(N-k));
                varcoef = (Xfe'*Xfe) \ total_sum / (Xfe'*Xfe);
            end
            
            % Compute the correct variance for the constant term
            varcoef(k,k) = mean(P*X)*varcoef*mean(P*X)';
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            if ~robust
                df_statistic = n*(T-1)-k+1;                  
            else
                df_statistic = n-1;              
            end
            p_statistic = (1 - tdist(abs(statistic), df_statistic))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald / (k-1);
            df_jointsig_n = k-1;
            if ~robust
                df_jointsig_d = n*(T-1)-k+1;
            else
                df_jointsig_d = n-1;
            end
            p_jointsig = 1 - fdist(jointsig, df_jointsig_n, df_jointsig_d);
            
            % Goodness of fit
            r2 = 1 - (res'*res)./(yfe'*yfe);
            %adjr2 = 1 - (N - 1)./(N - k) .* (1 - r2);
            adjr2 = NaN;
            
            % F-test individual effects [Baltagi(2008) Eq (2.12)]
            if ~robust
                res_pool = (y - X*((X'*X)\X'*y));
                RRSS = res_pool'*res_pool;
                URSS = res'*res;

                ieffectjointsig = ((RRSS - URSS)/(n-1)) / (URSS/(n*(T-1)-(k-1)));
                % Distributed as F
                df_ieffectjointsig_n = n-1;
                df_ieffectjointsig_d = N-n-k+1; % Same as n*(T-1)-(k-1)
                p_ieffectjointsig = 1 - fdist(ieffectjointsig, df_ieffectjointsig_n, df_ieffectjointsig_d);
            
                % Specific output for 'fe'
                out.ieffect = ieffect;
                out.ieffectjointsig = ieffectjointsig;
                out.df_ieffectjointsig_n = df_ieffectjointsig_n;
                out.df_ieffectjointsig_d = df_ieffectjointsig_d;
                out.p_ieffectjointsig = p_ieffectjointsig;
                
            end
                
            
            
        % Between estimation
        case 'be'
            
            % Add constant
            X = [X ones(N,1)];
            
            % Number of variables
            k = size(X,2);
            
            % Transform model
            ybe = sqrt(T) * P * y;
            Xbe = sqrt(T) * P * X;
            
            % Get only one observation per group
            ybe = ybe(1:T:end);
            Xbe = Xbe(1:T:end,:);
            
            % Compute estimates
            coef = (Xbe'*Xbe)\Xbe'*ybe;
            
            % Fitted values
            yhat = Xbe*coef;
            
            % Residuals
            res = ybe - yhat;
            
            % Residual variance
            resvar = (res'*res)./(n-k);
            
            % Covariance matrix
            varcoef = resvar * inv(Xbe'*Xbe);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = n-k;
            p_statistic = (1 - tdist(abs(statistic), df_statistic))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald / (k-1);
            df_jointsig_n = k-1;
            df_jointsig_d = n-k;
            p_jointsig = 1 - fdist(jointsig, df_jointsig_n, df_jointsig_d);
            
            % Goodness of fit
            temp = corrcoef(ybe,yhat).^2;
            r2 = temp(1,2);
            adjr2 = NaN;
            
            
        % Random-Effects GLS estimation: Swamy and Arora
        case 're'
            
            % Get sigm2_v from 'fe' residuals
            estfe = panel(yorig,Xorig,T,'fe');
            sigma2_v = estfe.resvar;
            
            % Get sigma2_1 from 'be' residuals
            estbe = panel(yorig,Xorig,T,'be');        
            sigma2_1 = estbe.resvar;
            
            % sigma2_u and rho_var
            sigma2_mu = (sigma2_1 - sigma2_v)/T;
            % correct sigma2_mu if 0 or < 0
            if sigma2_mu < 0
                sigma2_mu = 0;
                sigma2_mu_corrected = 1;
            else
                sigma2_mu_corrected = 0;
            end
            rho_mu = sigma2_mu /(sigma2_mu + sigma2_v);
            
            % Theta
            % theta  = 1 - (sqrt(sigma2_v) / sqrt(sigma2_1));  
            theta = 1- sqrt(sigma2_v/(T*sigma2_mu + sigma2_v));
            
            % Add constant
            X = [X ones(N,1)];
            
            % Transform the model            
            Qre = eye(N) - theta * P;
            yre = Qre*y;
            Xre = Qre*X;
            
            % Number of variables
            k = size(Xre,2);
            
            % Compute estimates
            coef = (Xre'*Xre)\Xre'*yre;
            
            % Fitted values
            yhat = Xre*coef;
            
            % Residuals
            res = yre - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N-k);
            
            % Covariance matrix
            if ~robust
                varcoef = resvar * inv(Xre'*Xre);
            else
                total_sum = 0;
                for i=1:1:n
                    s = (i-1)*T+1;
                    e = i*T;
                    total_sum = total_sum + Xre(s:e,:)'*res(s:e)*res(s:e)'*Xre(s:e,:);
                end
                % Adjust Std.errors for n clusters
                total_sum = total_sum .* (n/(n-1)) * ((N-1)/(N-k));
                varcoef = (Xre'*Xre) \ total_sum / (Xre'*Xre);
            end
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic))) * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            temp = corrcoef(yhat, yre).^2;
            r2 = temp(1,2);
            adjr2 = NaN;
            
            % Specific output for 're'
            out.theta = theta;
            out.sigma_v = sqrt(sigma2_v);
            out.sigma_1 = sqrt(sigma2_1);
            out.sigma_mu = sqrt(sigma2_mu);
            out.sigma_mu_corrected = sigma2_mu_corrected;
            out.rho_mu = rho_mu;
            
        otherwise
            error('Incorrect method especified');
                        
    end
    
    % Save output
    out.method = 'Panel';
    out.method2 = method;
    out.y = y;
    out.X = X;    
    out.N = N;
    out.n = n;
    out.T = T;
    out.k = k;
    out.constant = 1;
    out.robust = robust;
    out.coef = coef;
    out.varcoef = varcoef;
    out.stderr = stderr;
    out.yhat = yhat;
    out.res = res;
    out.resvar = resvar;
    out.statistic = statistic;
    out.df_statistic = df_statistic;
    out.p_statistic = p_statistic;
    out.jointsig = jointsig;
    out.df_jointsig_n = df_jointsig_n;
    out.df_jointsig_d = df_jointsig_d;
    out.p_jointsig = p_jointsig;
    out.r2 = r2;
    out.adjr2 = adjr2;
    


end

