function [ out ] = s2sls( y, X, L, W )
%S2SLS Spatial Two Stage Least Squares estimation
%   est = S2SLS(Y,X,L,W,OPTIONS) computes instrumental variables 2SLS
%   estimation where Y is the vector of the dependant variable, X is the 
%   matrix of the independent variables, L is the matrix of spatial lagged
%   independent variables, and W is the contiguity matrix.
%
%   L is optional and can be set to [] if no spatial independent variables
%   in the model. Variables in L will be added as W*L to X but not to 
%   instruments H.
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    % Check input
    if nargin < 1
        error('Dependent variable not specified');
    end
    if nargin < 2
        error('Independent variables not specified');
    end
    if size(y,2) ~= 1
        error('Y must be a column vector of the dependant variable');
    end
    if size(y,1) ~= size(X,1)
        error('Number of rows in Y must be equal to number of rows in X');
    end
    if size(L,1) > 0
        if size(L,1) ~= size(X,1)
            error('Number of rows in L must be equal to number of rows in X');
        end
    end
    if nargin < 4
        error('W Matrix not specified');
    end
    if size(W) ~= [size(y,1) size(y,1)]
        error('W must be a square matrix');
    end

    
    % Output structure
    out = estoutput();

    % Get number of observations
    n = size(y,1);
    N = n;    
       
    % Build Instruments   
    H = [X, W*X, W^2*X, ones(N,1)];
    
    % Add laggedd independent variables
    if L
        X = [X, W*L];
    end
    
    % Build Z
    Z = [X, W*y, ones(N,1)];

    % Number of variables and instruments
    k = size(Z,2);
    l = size(H,2);
    
    % First Stage
    Zhat = H*((H'*H)\H'*Z);

    % Compute estimates: Second Stage
    coef = (Zhat'*Z)\Zhat'*y;
    
    % Fitted values
    % yhat = Z*coef;
    yhat = inv(eye(N) - coef(end-1)*W) * ( [Z(:,1:end-2) Z(:,end)] * [coef(1:end-2); coef(end)] );

    % Residuals
    res = y - yhat;

    % Residual variance
    resvar = (res'*res)./(N);

    % Covariance matrix
    varcoef = resvar * inv(Zhat'*Zhat);

    % Standard error
    stderr = sqrt(diag(varcoef));

    % Statistic
    statistic = coef ./ stderr;
    df_statistic = NaN;
    p_statistic = (1 - normaldist(abs(statistic)))' * 2;

    % Joint significance
    R = [eye(k-1) zeros(k-1,1)];
    q = zeros(k-1,1);
    W = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
    jointsig = W;
    df_jointsig_n = k-1;
    df_jointsig_d = Inf;
    p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);

    % Goodness of fit
    r2 = NaN;
    adjr2 = NaN;
    
    % Save output output
    out.method = 'S2SLS';
    out.method2 = '';
    out.y = y;
    out.X = Z;  % Store also the spatial lagged dependent variable
    out.H = H;
    out.N = N;
    out.n = n;
    out.T = 1;
    out.k = k;
    out.l = l;
    out.constant = 1;
    out.robust = 0;
    out.coef = coef;
    out.varcoef = varcoef;
    out.stderr = stderr;
    out.yhat = yhat;
    out.res = res;
    out.resvar = resvar;
    out.statistic = statistic;
    out.df_statistic = df_statistic;
    out.p_statistic = p_statistic;
    out.jointsig = jointsig;
    out.df_jointsig_n = df_jointsig_n;
    out.df_jointsig_d = df_jointsig_d;
    out.p_jointsig = p_jointsig;
    out.r2 = r2;
    out.adjr2 = adjr2;
    

end

