function [ out ] = spanel( y, X, L, W, T, method )
%PANEL Spatial Panel data estimation
%   est = SPANEL(Y,X,L,W,T,HMETHOD) computes panel data estimation where Y is the
%   vector of the dependant variable, X is the matrix of the independent
%   variables, L is the matrix of spatial lagged independent variables, 
%   W is the contiguity matrix, T is the number of observations per individual,
%   and method is a string for the estimation method to use.
%   Possible values for METHOD are
%
%    'po' Spatial Pooled estimation
%    'fe' Spatial Fixed-Effects (Within) estimation
%    'be' Spatial Between estimation
%    're' Spatial Random-Effects GLS estimation: Swamy and Arora
%    'ec' Spatial Error Components estimation
%    'bs' Best Spatial 2SLS estimator
%    'sec-b' Spatial Error Components Best 2SLS estimator
%
%   L is optional and can be set to [] if no spatial independent variables
%   in the model. Variables in L will be added as W*L to X but not to 
%   instruments H.
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    % Check input
    if nargin < 1
        error('Dependent variable not specified');
    end
    if nargin < 2
        error('Independent variables not specified');
    end
    if size(y,2) ~= 1
        error('Y must be a column vector of the dependant variable');
    end
    if size(y,1) ~= size(X,1)
        error('Number of rows in Y must be equal to number of rows in X');
    end
    if size(L,1) > 0
        if size(L,1) ~= size(X,1)
            error('Number of rows in L must be equal to number of rows in X');
        end
    end
    if nargin < 4
        error('W matrix not specified');
    end
    if size(W,1) ~= size(W,2)
        error('W must be a square matrix');
    end
    if size(W,1) ~= size(y,1)
        error('Number of rows in W must be equal to number of rows in y and X');
    end
    if nargin < 5
        error('T not specified');
    end
    if length(T) ~= 1
        error('T must be a scalar indicating the number of observations per group');
    end
    if nargin < 6
        warning('METHOD not specified. Using ''fe'' by default');
        method = 'fe';
    end
    
    % Output structure
    out = estoutput();

    % Get number of observations and groups
    N = size(y,1);
    n = N/T;
    
    % Usefull matrices
    Jbar = ones(T)./T;
    E = eye(T) - Jbar;  % Matrix that compute differences from group mean
    P = kron(eye(n),Jbar); % Compute group means and replicate for all observations in the group
    Q = eye(N) - P; % Compute differences from group means
    
    % Original variables
    yorig = y; % Original y
    Xorig = X; % Original X
    Lorig = L; % Original L
    
    % Estimate
    switch(method)
        
        % Pooled estimation
        case 'po'         
            
            % Build Instruments   
            H = [X, W*X, W^2*X, ones(N,1)];
            
            % Add laggedd independent variables
            if L
                X = [X, W*L];
            end
            
            % Build Z
            Z = [X, W*y, ones(N,1)];
            
            % Number of variables and instruments
            k = size(Z,2);
            l = size(H,2);

            % First Stage
            Zhat = H*((H'*H)\H'*Z);
            
            % Compute estimates: Second Stage
            coef = (Zhat'*Z)\Zhat'*y;
            
            % Fitted values
            % yhat = Z*coef;
            yhat = inv(eye(N) - coef(end-1)*W) * ( [Z(:,1:end-2) Z(:,end)] * [coef(1:end-2); coef(end)] );
            
            % Residuals
            res = y - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N);
            
            % Covariance matrix
            varcoef = resvar * inv(Zhat'*Z);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic)))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            r2 = NaN;
            adjr2 = NaN;
            
        % Fixed-Effects (Within) estimation
        case 'fe'          
                         
            % Build Instruments   
            H = [X, W*X, W^2*X];
            
            % Add laggedd independent variables
            if L
                X = [X, W*L];
            end
            
            % Build Z
            Z = [X, W*y];  
            
            % Add constant and transform variables
            Zfe = [Q*Z ones(N,1)];
            Z = [Z ones(N,1)];
            Hfe = [Q*H ones(N,1)];
            H = [H ones(N,1)];
            yfe = Q*y;          

            % Number of variables and instruments
            k = size(Z,2);
            l = size(Hfe,2);
            
            % First Stage
            Zfehat = Hfe*((Hfe'*Hfe)\Hfe'*Zfe);
            
            % Compute estimates: Second Stage
            coef = (Zfehat'*Zfe)\Zfehat'*yfe;
            
            % Compute the correct constant term
            alpha = mean(P*y) - mean(P*Z*coef);
            coef(k) = alpha;
            
            % Compute individual effects
            ieffect = P*(y-Z*coef);
            
            % Fitted values
            % yhat = Z*coef + ieffect;
            yhat = inv(eye(N) - coef(end-1)*W) * ( [Z(:,1:end-2) Z(:,end)] * [coef(1:end-2); coef(end)] + ieffect);
            
            % Residuals
            res = y - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N-k-n+1);
            
            % Covariance matrix
            varcoef = resvar * inv(Zfehat'*Zfe);
            
            % Compute the correct variance for the constant term
            varcoef(k,k) = mean(P*Z)*varcoef*mean(P*Z)';
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic)))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            r2 = NaN;
            adjr2 = NaN;
                        
            % Specific output for 'fe'
            out.ieffect = ieffect;
           
                       
            
        % Between estimation
        case 'be'
            
            % Build Instruments   
            H = [X, W*X, W^2*X, ones(N,1)];
            
            % Add laggedd independent variables
            if L
                X = [X, W*L];
            end
            
            % Build Z
            Z = [X, W*y, ones(N,1)];
            
            % Number of variables
            k = size(Z,2);
            l = size(H,2);
            
            % Transform model
            ybe = sqrt(T) * P * y;
            Zbe = sqrt(T) * P * Z;
            Hbe = sqrt(T) * P * H;
            
            % Get only one observation per group
            ybe = ybe(1:T:end);
            Zbe = Zbe(1:T:end,:);
            Hbe = Hbe(1:T:end,:);
            
            % First Stage
            Zbehat = Hbe*((Hbe'*Hbe)\Hbe'*Zbe);                     
            
            % Compute estimates: Second Stage
            coef = (Zbehat'*Zbe)\Zbehat'*ybe;
            
            % Fitted values
            % yhat = Zbe*coef;
            invterm = inv(eye(N) - coef(end-1)*W);  
            invterm2 = NaN(n);
            for i = 1:1:n
                for j = 1:1:n
                    invterm2(i,j) = mean(diag(invterm((i-1)*T+1:i*T, (j-1)*T+1:j*T)));
                end
            end
            yhat = invterm2 * ( [Zbe(:,1:end-2) Zbe(:,end)] * [coef(1:end-2); coef(end)] );
            
            % Residuals
            res = ybe - yhat;
            
            % Residual variance
            resvar = (res'*res)./(n-k);
            
            % Covariance matrix
            varcoef = resvar * inv(Zbehat'*Zbe);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic)))' * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            W = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = W;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            r2 = NaN;
            adjr2 = NaN;
            
        % Random-Effects GLS estimation: Swamy and Arora
        case {'re','ec','bs','sec-b'}  
            
            % Build instruments
            H = [X, W*X, W^2*X];            
                       
            % Add laggedd independent variables
            if L     
                X = [X, W*L];                
            end
            
            % Build Z
            Z = [X, W*y];
                                                           
            % Get sigm2_v from 'fe' residuals   % WITHOUT LEE INSTRUMENTS         
            estfe = spanel(yorig,Xorig,Lorig,W,T,'fe');
            sigma2_v = estfe.resvar;
            
            % Get sigme2_1 from 'be' residuals   % WITHOUT LEE INSTRUMENTS       
            estbe = spanel(yorig,Xorig,Lorig,W,T,'be'); 
            sigma2_1 = estbe.resvar;
            
            % sigma2_u and rho_var
            sigma2_mu = (sigma2_1 - sigma2_v)/T;
            % correct sigma2_u if 0 or < 0
            if sigma2_mu < 0
                sigma2_mu = 0;
                sigma2_mu_corrected = 1;
            else
                sigma2_mu_corrected = 0;
            end
            rho_mu = sigma2_mu /(sigma2_mu + sigma2_v);  
            
            % Theta 
            theta = 1- sqrt(sigma2_v/(T*sigma2_mu + sigma2_v));
            
            % If Lee best instruments
            if strcmp(method,'bs') || strcmp(method,'sec-b')    
                estpo = spanel(y,Xorig,Lorig,W,T,'po');
                lambda = estpo.coef(end-1);
                beta = estpo.coef(1:end-2);
                A = eye(n*T) - lambda*W;  
                H = [X, W*inv(A)*X*beta];
            end
            
            % If error component 
            if strcmp(method,'ec') || strcmp(method,'sec-b');
                H = [Q*H, P*H];
            end
            
            % Add constant
            Z = [Z ones(N,1)];
            H = [H ones(N,1)];
            
            % Transform the model
            Qre = eye(N) - theta * P;
            yre = Qre*y;
            Zre = Qre*Z;
            Hre = Qre*H;
            
            % Number of variables
            k = size(Zre,2);
            l = size(Hre,2);
            
            % First stage
            Zrehat = Hre*((Hre'*Hre)\Hre'*Zre);
            
            % Compute estimates: Second stage
            coef = (Zrehat'*Zre)\Zrehat'*yre;
            
            % Fitted values
            % yhat = Zre * coef;
            yhat = inv(eye(N) - coef(end-1)*W) * ( [Zre(:,1:end-2) Zre(:,end)] * [coef(1:end-2); coef(end)] );
            
            % Residuals
            res = yre - yhat;
            
            % Residual variance
            resvar = (res'*res)./(N-k);
            
            % Covariance matrix
            varcoef = resvar * inv(Zrehat'*Zre);
            
            % Standard error
            stderr = sqrt(diag(varcoef));
            
            % Statistic
            statistic = coef ./ stderr;
            df_statistic = NaN;
            p_statistic = (1 - normaldist(abs(statistic))) * 2;

            % Joint significance
            R = [eye(k-1) zeros(k-1,1)];
            q = zeros(k-1,1);
            Wald = (R*coef - q)' * inv(R*varcoef*R')*(R*coef - q); 
            jointsig = Wald;
            df_jointsig_n = k-1;
            df_jointsig_d = Inf;
            p_jointsig = 1 - chi2dist(jointsig, df_jointsig_n);
            
            % Goodness of fit
            r2 = NaN;
            adjr2 = NaN;
            
            % Specific output for 're'
            out.theta = theta;
            out.sigma_v = sqrt(sigma2_v);
            out.sigma_1 = sqrt(sigma2_1);
            out.sigma_mu = sqrt(sigma2_mu);
            out.sigma_mu_corrected = sigma2_mu_corrected;
            out.rho_mu = rho_mu;
            
        otherwise
            error('Incorrect method especified');
                        
    end
    
    % Save output
    out.method = 'SPanel';
    out.method2 = method;
    out.y = y;
    out.X = Z;  % Store also the spatial lagged dependent variable
    out.H = H;
    out.N = N;
    out.n = n;
    out.T = T;
    out.k = k;
    out.l = l;
    out.constant = 1;
    out.robust = 0;
    out.coef = coef;
    out.varcoef = varcoef;
    out.stderr = stderr;
    out.yhat = yhat;
    out.res = res;
    out.resvar = resvar;
    out.statistic = statistic;
    out.df_statistic = df_statistic;
    out.p_statistic = p_statistic;
    out.jointsig = jointsig;
    out.df_jointsig_n = df_jointsig_n;
    out.df_jointsig_d = df_jointsig_d;
    out.p_jointsig = p_jointsig;
    out.r2 = r2;
    out.adjr2 = adjr2;
    


end

