function [ H , p ] = hausman( estA, estB )
%HAUSMAN Hausman's test of specification
%   [H , P] = HAUSMAN( ESTA, ESTB ) performs Hausman's test of
%   specification being ESTA and ESTB consistent under the null hypothesis,
%   and ESTA consisting under the alternative hypothesis. The value of the
%   test is returned in H and the p-vaue in P.
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    if estA.robust || estB.robust
        error('Hausman''s test not valid for robust estimations');
    end
    
    if estA.k ~= estB.k
        error('Number of estimated coefficients must be the same in both estimations');
    end
    
    % Get coefficients and covariance matrices
    k = estA.k;
    
    ktest = k;
    
    coefA = estA.coef(1:ktest-1);
    coefB = estB.coef(1:ktest-1);
    varcoefA = estA.varcoef(1:ktest-1,1:ktest-1);
    varcoefB = estB.varcoef(1:ktest-1,1:ktest-1);
    
    
    % Compute Hausman's statistic
    H = (coefA - coefB)' * inv(varcoefA - varcoefB) * (coefA - coefB);
    
    if H < 0
        error('Invalid estimates. H < 0');
    end
    
    % Distributed as a Chi-Square with ktest degrees of freedom
    p = 1 - chi2dist(H, ktest-1);
    

end

