function [  ] = estprint( est )
%ESTPRINT Prints estimation results
%   ESTPRINT(EST) prtins estimation results
%
%   AUTHOR:
%   Inmaculada C. lvarez
%   Javier Barbero
%   Jos L. Zofo
%
%   http://www.paneldatatoolbox.com
%
%   Version: 1.0
%   LAST UPDATE: 17, October, 2013
%

    isSpatial = 0;

    fprintf('_________________________________________________________\n');

    switch(est.method)
        case 'OLS'
            fprintf('<strong>Ordinary Least Squares</strong>\n');
        case 'IV2SLS'
            fprintf('<strong>Instrumental Variables 2SLS</strong>\n');
        case 'S2SLS'
            fprintf('<strong>Spatial Two Stage Least Squares (S2SLS)</strong>\n');
            isSpatial = 1;
        case 'Panel'                        
            switch(est.method2)
                case 'po'
                    fprintf('<strong>Panel: Pooled OLS</strong>\n');
                case 'fe'
                    fprintf('<strong>Panel: Fixed effects (Within)</strong>\n');
                case 'be'
                    fprintf('<strong>Panel: Between effects</strong>\n');
                case 're'
                    fprintf('<strong>Panel: Random effects GLS (Swamy and Arora)</strong>\n');
            end
        case 'IVPanel'
            switch(est.method2)
                case 'po'
                    fprintf('<strong>IV Panel: Pooled OLS</strong>\n');
                case 'fe'
                    fprintf('<strong>IV Panel: Fixed effects (Within)</strong>\n');
                case 'be'
                    fprintf('<strong>IV Panel: Between effects</strong>\n');
                case 're'
                    fprintf('<strong>IV Panel: Random effects GLS (Swamy and Arora)</strong>\n');
                case 'ec'
                    fprintf('<strong>IV Panel: Error components (EC2SLS)</strong>\n');
            end
        case 'SPanel'
            isSpatial = 1;
            switch(est.method2)
                case 'po'
                    fprintf('<strong>Spatial Panel: Pooled S2SLS</strong>\n');
                case 'fe'
                    fprintf('<strong>Spatial Panel: Fixed effects (FE-2SLS)</strong>\n');
                case 'be'
                    fprintf('<strong>Spatial Panel: Between effects (BE-2SLS)</strong>\n');
                case 're'
                    fprintf('<strong>Spatial Panel: Random effects (RE-2SLS)</strong>\n');
                case 'ec'
                    fprintf('<strong>Spatial Panel: Error Components (SEC-2SLS)</strong>\n');
                case 'bs'
                    fprintf('<strong>Spatial Panel: Best (BS-2SLS)</strong>\n');
                case 'sec-b'
                    fprintf('<strong>Spatial Panel: Error Components Best (SEC-B2SLS)</strong>\n');
            end
        otherwise
            error('Unknown method');
    end
    
    disp(' ');
    fprintf('N observations: %d \n',est.N);
    
    switch(est.method)
        case {'Panel','IVPanel','SPanel'}
            fprintf('N groups: %d \n',est.n);
            fprintf('Obs per group: %d \n',est.T);
    end
    
    if ~isnan(est.r2)
        fprintf('R-squared = %f \n',est.r2);
    end
    if ~isnan(est.adjr2)
        fprintf('Adj R-squared = %f \n',est.adjr2);
    end
    
    if est.df_jointsig_d == Inf
        % Is Chi-Square
        fprintf('Joint significance: Chi2(%d) = %f  \n', est.df_jointsig_n, est.jointsig);
        fprintf('                    p-value = %5.4f \n',est.p_jointsig);
    else
        % Is F
        fprintf('Joint significance: F(%d, %d) = %f \n', est.df_jointsig_n, est.df_jointsig_d, est.jointsig);
        fprintf('                    p = %5.4f \n', est.p_jointsig);
    end
    
    % Add dependent variable if not specified
    if isempty(est.dvarnames) || ~iscell(est.dvarnames)
        est.dvarnames = {'deptvar'};
    end
    
    fprintf('Dept Var: %s\n',char(est.dvarnames));
        
    if (strcmp(est.method2,'fe') || strcmp(est.method2,'re')) && est.robust
        fprintf('Robust standard errors adjusted for %d clusters \n',est.n);
    elseif est.robust
        fprintf('Robust standard errors \n');
    end
    
    %disp(' ');
    
    est = adddefivarnames(est);
    
    
    fprintf('--------------------------------------------------------------\n');
    disp('       Varname  Coefficient   Std. Error  Statistic  p-value');
    fprintf('--------------------------------------------------------------\n');
    for i=1:1:est.k   
        fprintf('%14.14s  %11.5f  %10.5f  %9.4f  %7.3f',char(est.ivarnames(i)),est.coef(i),est.stderr(i),est.statistic(i), est.p_statistic(i));
        
        % p-value stars
        if est.p_statistic(i) < 0.01
            fprintf('***');
        elseif est.p_statistic(i) < 0.05
            fprintf('** ');
        elseif est.p_statistic(i) < 0.10
            fprintf('*  ');
        end
        
        % New line
        fprintf('\n');
    end
    fprintf('--------------------------------------------------------------\n');
    
    %disp(' ');
    
    % Test of individual effects (Panel data 'fe');
    if strcmp(est.method2,'fe') && strcmp(est.method,'Panel') && ~est.robust
        fprintf(' Test of individual effects: F(%d,%d) = %f \n',est.df_ieffectjointsig_n, est.df_ieffectjointsig_d, est.ieffectjointsig);
        fprintf('                             p-value = %4.3f \n',est.p_ieffectjointsig);
    end
    
    % Panel data 're'
    if strcmp(est.method2,'re') || strcmp(est.method2,'ec') || strcmp(est.method2,'bs') || strcmp(est.method2,'sec-b')
        if est.sigma_mu_corrected
            fprintf(' sigma_mu = %7.6f (Replaced by zero)\n', est.sigma_mu)
        else
            fprintf(' sigma_mu = %7.6f \n', est.sigma_mu)
        end
        fprintf('  sigma_v = %7.6f     sigma_1 = %7.6f \n', est.sigma_v, est.sigma_1)
        fprintf('   rho_mu = %7.6f       Theta = %7.6f \n', est.rho_mu, est.theta)
        %fprintf('Theta = %f \n',est.theta);
    end

    disp(' ');

end

